import copy

from SlatSystem import *
from slat_generator import AssignSlatsFromPool, GetIncorrectBindingStatsForSlats, slat_color_mapping
import sys
import getopt


color_list = ["white", "green", "red", "blue", "yellow", "pink", "aqua", "lime", "olive", "orange", "gold", "grey", "purple", "maroon", "turquoise", "black"]
# The following colors aren't in the standard wxPython database so the listed conversions will be made for viewing in SlatSimulator
wxPython_color_mapping = {'aqua': 'AQUAMARINE', 'lime': 'LIME GREEN', 'olive': 'KHAKI'}


#["help", "coop=", "dup=", "rect", "pdf", "svg", "out="]
def DisplayHelp():
    print("\nThis script outputs a set of slats that self-assemble a ribbon with properties based on the")
    print("following parameters.")
    print("\nUsage:")
    print("-h, --help\t\t\t\t\tDisplays this help message.")
    print("-c <cooperativity>, --coop=<cooperativity>\t[Optional] Cooperativity of slats. Defaults to 4.")
    print("-d <duplicity>, --dup=<duplicity>\t\t[Optional] Duplicity of slats (i.e. number of unique slats per layer).")
    print("\t\t\t\t\t\t Defaults to the cooperativity value.")
    print("-n <num_handles>, --num=<num_handles>\t\t[Optional] If specified, the number of unique handles to be randomly assigned.")
    print("\t\t\t\t\t\t  Otherwise, all glue locations will use unique handles.")
    print("-e <num_errors>, --errors=<num_errors>\t\t[Optional] If specified, the maximum binding error for an erroneous slat binding.")
    print("\t\t\t\t\t\t  The produced system will have at least one slat capable of this error.")
    print("-i, --iter\t\t\t\t\t[Optional] If specified, the maximum number of iterations to attempt to find each system.")
    print("\t\t\t\t\t\t  If not specified, defaults to 1000.")
    print("-r, --rect\t\t\t\t\t[Optional] If specified, the rectangular slat pattern will be used.")
    print("\t\t\t\t\t\t  Otherwise, the staggered pattern will be used. WARNING: staggered is not fully supported.")
    print("-p, --pdf\t\t\t\t\t[Optional] If this option is used, the slats will be output to a PDF file.")
    print("-s, --svg\t\t\t\t\t[Optional] If this option is used, the slats will be output to an SVG file.")
    print("-o <dir>, --out=<dir>\t\t\t\t[Optional] Directory for output files. If not specified,")
    print("\t\t\t\t\t\t the current directory will be used.")
    print("-m <slat pattern>, --motif=<slat pattern>\t[Optional] If specified, a list of characters denoting the slat type")
    print("\t\t\t\t\t\t repetition pattern for each macrotile, e.g. [aabbababbbabaaab].")
    print("-l <motif file>, --listfile=<motif file>\t[Optional] A file containing a list of motifs to be used.")
    print("-f <T/F>, --flip=<T/F>\t\t\t\t[Optional] If this option is set to 'F', versions of slat types rotated 180 degrees")
    print("\t\t\t\t\t\t will not be included. Otherwise, they will.")
    print("")
    exit()


def GetSlatsForMotif(motif):

    print(f'Making slats for motif {motif}...')

    horiz_slat_types = dict()
    vert_slat_types = dict()
    slat_labels = set()

    motif_types = list(set(motif))
    motif_types.sort()

    # Make horizontal slats
    horiz_slat_num = 0
    for t in motif_types:
        blocks = []
        domains = {}
        loc = 0
        for i in range(len(motif)):  # horizontal input/vertical output labels
            label = f'{t}{motif[i]}1'
            coords = (loc, 0, 0)
            domains[coords] = [(label, 'D', 1)]
            blocks.append(coords)
            slat_labels.add(label)
            loc += 1

        loc = len(motif)
        for o in range(len(motif)):  # horizontal output/vertical input labels
            label = f'{motif[o]}{t}2'
            coords = (loc, 0, 0)
            domains[coords] = [(label, 'D', 1)]
            blocks.append(coords)
            slat_labels.add(label)
            loc += 1

        slat_type = SlatType(name=f'h-{t}', blocks=blocks, domains=domains)
        # print(f'horizontal slat_type: {slat_type}')
        slat_type.color = color_list[horiz_slat_num % len(color_list)]
        horiz_slat_types[t] = slat_type
        horiz_slat_num += 1

    # Make vertical slats
    vert_slat_num = 0
    for t in motif_types:
        blocks = []
        domains = {}
        loc = 0
        for i in range(len(motif)):  # horizontal input/vertical output labels
            label = f'{motif[i]}{t}1*'
            coords = (0, loc, 0)
            domains[coords] = [(label, 'U', 1)]
            blocks.append(coords)
            slat_labels.add(label)
            loc += 1

        loc = len(motif)
        for o in range(len(motif)):  # horizontal output/vertical input labels
            label = f'{t}{motif[o]}2*'
            coords = (0, loc, 0)
            domains[coords] = [(label, 'U', 1)]
            blocks.append(coords)
            slat_labels.add(label)
            loc += 1

        slat_type = SlatType(name=f'v-{t}', blocks=blocks, domains=domains)
        # print(f'vertical slat_type: {slat_type}')
        slat_type.color = color_list[vert_slat_num % len(color_list)]
        vert_slat_types[t] = slat_type
        vert_slat_num += 1

    # pruned_slat_labels = []
    # for slat_label in slat_labels:
    #     if not slat_label[-1] == '*':
    #         pruned_slat_labels.append(slat_label)
    # pruned_slat_labels.sort()
    # print(f'Slat set has the following {len(pruned_slat_labels)} labels (not counting complements): {pruned_slat_labels}')

    (max_half_self_overlap, max_self_overlap, max_cross_overlap,
     max_half_self_overlap_count, max_self_overlap_count, max_cross_overlap_count,
     max_half_self_overlap_example, max_self_overlap_example, max_cross_overlap_example) = GetIncorrectBindingStatsForSlats(horiz_slat_types, vert_slat_types)


    results_dict = {'max_half_self_overlap': max_half_self_overlap, 'max_self_overlap': max_self_overlap, 'max_cross_overlap': max_cross_overlap,
                    'max_half_self_overlap_count': max_half_self_overlap_count, 'max_self_overlap_count': max_self_overlap_count, 'max_cross_overlap_count': max_cross_overlap_count,
                    'horiz_slat_types': horiz_slat_types, 'vert_slat_types': vert_slat_types,
                    'max_half_self_overlap_example': max_half_self_overlap_example, 'max_self_overlap_example': max_self_overlap_example, 'max_cross_overlap_example': max_cross_overlap_example}
    return results_dict

def is_subsequence(subsequence, sequence):
    shift_range = len(sequence)-len(subsequence)+1
    for i in range(shift_range):
        if sequence[i:i+len(subsequence)] == subsequence:
            return i
    return -1

def GetShiftedSlatsForOverlap(example_slat1, example_subslat1, example_slat2, example_subslat2):
    noncomp_slat1 = copy.deepcopy(example_slat1)
    noncomp_slat2 = copy.deepcopy(example_slat2)
    slat1_flip = False
    slat2_flip = False
    if ('*' in noncomp_slat1[0]):
        if (not '*' in noncomp_slat2[0]):
            noncomp_slat1 = [a[:-1] for a in noncomp_slat1]
    elif ('*' in noncomp_slat2[0]):
        noncomp_slat2 = [a[:-1] for a in noncomp_slat2]

    shifted_slat1 = copy.deepcopy(example_slat1)
    slat1_shift = is_subsequence(example_subslat1, noncomp_slat1)
    if slat1_shift == -1:
        slat1_shift = is_subsequence(example_subslat1, noncomp_slat1[::-1])
        shifted_slat1 = shifted_slat1[::-1]
        slat1_flip = True
    if slat1_shift == -1:
        print(f'ERROR: no shift found for {example_subslat1} {example_slat1}')
    else:
        shifted_slat1 = ['___' for i in range(slat1_shift)] + shifted_slat1
        # print(f'shift: {shift} for {max_self_example_slat1}:\n{shifted_slat1}')

    shifted_slat2 = copy.deepcopy(example_slat2)
    slat2_shift = is_subsequence(example_subslat2, noncomp_slat2)
    if slat2_shift == -1:
        slat2_shift = is_subsequence(example_subslat2, noncomp_slat2[::-1])
        shifted_slat2 = shifted_slat2[::-1]
        slat2_flip = True
    if slat2_shift == -1:
        print(f'ERROR: no shift found for {example_subslat2} {shifted_slat2}')
    else:
        shifted_slat2 = ['___' for i in range(slat2_shift)] + shifted_slat2
        # print(f'shift: {shift} for {max_self_example_slat2}:\n{shifted_slat2}')
    return (slat1_shift, slat2_shift, slat1_flip, slat2_flip, shifted_slat1, shifted_slat2)

def RibbonSystemGenerator(options_dict):

    slat_color_mapping = options_dict['slat_color_mapping']
    cooperativity = options_dict['cooperativity']
    duplicity = options_dict['duplicity']
    b_staggered = options_dict['b_staggered']
    growth_pattern = options_dict['growth_pattern']
    b_pdf = options_dict['b_pdf']
    b_svg = options_dict['b_svg']
    directory = options_dict['directory']
    num_handles = options_dict['num_handles']
    max_errors = options_dict['max_errors']
    max_iterations = options_dict['max_iterations']
    motif = options_dict['motif']
    motif_list = options_dict['motif_list']
    b_add_flipped_slats = options_dict['b_add_flipped_slats']

    max_self_overlap = None
    max_cross_overlap = None
    slat_system_errors = None

    if not motif is None:
        if motif_list is None:
            motif_list = [motif]

        if not len(motif) == cooperativity:
            print(f'ERROR: Motif must have length equal to the cooperativity value.')
            exit()

        slat_types = list(set(motif))

        if duplicity is None:
            duplicity = len(slat_types)

        if duplicity != len(slat_types):
            print(f'ERROR: If both duplicity and motif arguments are given, the duplicity must equal the number of slat types contained in the motif.')
            exit()

    elif duplicity is None:
        duplicity = cooperativity

    error_string = ''
    if not max_errors is None:
        error_string = f'-errors{max_errors}'

    slat_system = SlatSystem()
    slat_system.b_use_rotation = False
    slat_system.binding_threshold = cooperativity
    slat_length = cooperativity * 2

    ######################################
    # Rectangular ribbon growth pattern
    if b_staggered == False:

        motif_results_dict = dict()
        motif_results_list = []

        b_keep_going = True
        iteration_count = 0
        #while b_keep_going:
        for motif in motif_list:

            #random.shuffle(motif)
            #print(f'shuffled motif: {motif}')

            slat_types = list(set(motif))
            duplicity = len(slat_types)
            motif_str = ''.join(map(str, motif))
            #print(f'motif: {motif_str}')

            ret_dict = GetSlatsForMotif(motif)
            motif_results_dict[motif_str] = ret_dict
            #curr_max_self_overlap = ret_dict['max_self_overlap']
            #curr_max_cross_overlap = ret_dict['max_cross_overlap']
            motif_results_list.append((ret_dict['max_self_overlap'], ret_dict['max_half_self_overlap'],  ret_dict['max_cross_overlap'],
                                       ret_dict['max_self_overlap_count'], ret_dict['max_half_self_overlap_count'], ret_dict['max_cross_overlap_count'], motif_str))
            # if ((curr_max_self_overlap < 3) and (curr_max_cross_overlap < 3)) or (iteration_count >= 5000):
            #     b_keep_going = False
            # else:
            #     if (iteration_count % 25) == 0:
            #         print(f'iteration_count: {iteration_count}')
            #     iteration_count += 1


        # Sort the motifs to find the 'best'
        motif_results_list.sort()
        print(f'motif_results_list (max auto, max cross, count max auto, count max cross, motif):')
        for motif_result in motif_results_list:
            print(f' {motif_result}')

        # Get the 'best' motif from the sorted list
        motif = motif_results_list[0][6]
        max_self_overlap = motif_results_dict[motif]['max_self_overlap'] #motif_results_list[0][0]
        #max_half_self_overlap = motif_results_dict[motif]['max_half_self_overlap']
        max_cross_overlap = motif_results_dict[motif]['max_cross_overlap'] #motif_results_list[0][1]
        horiz_slat_types = motif_results_dict[motif]['horiz_slat_types']
        vert_slat_types = motif_results_dict[motif]['vert_slat_types']

        max_self_overlap_example = motif_results_dict[motif]['max_self_overlap_example']
        max_self_example_slat1 = max_self_overlap_example[0]
        max_self_example_slat2 = max_self_overlap_example[1]
        max_self_example_subslat1 = max_self_overlap_example[2]
        max_self_example_subslat2 = max_self_overlap_example[3]
        (max_self_example_slat1_shift, max_self_example_slat2_shift,
         max_self_example_slat1_flip, max_self_example_slat2_flip,
         shifted_max_self_slat1, shifted_max_self_slat2) = GetShiftedSlatsForOverlap(max_self_example_slat1,
                                                                                     max_self_example_subslat1,
                                                                                     max_self_example_slat2,
                                                                                     max_self_example_subslat2)

        max_cross_overlap_example = motif_results_dict[motif]['max_cross_overlap_example']
        max_cross_example_slat1 = max_cross_overlap_example[0]
        max_cross_example_slat2 = max_cross_overlap_example[1]
        max_cross_example_subslat1 = max_cross_overlap_example[2]
        max_cross_example_subslat2 = max_cross_overlap_example[3]
        (max_cross_example_slat1_shift, max_cross_example_slat2_shift,
         max_cross_example_slat1_flip, max_cross_example_slat2_flip,
         shifted_max_cross_slat1, shifted_cross_self_slat2) = GetShiftedSlatsForOverlap(max_cross_example_slat1,
                                                                                        max_cross_example_subslat1,
                                                                                        max_cross_example_slat2,
                                                                                        max_cross_example_subslat2)

        print(f'best motif: {motif_results_list[0]}')#,\nmax self overlap example: {max_self_overlap_example}\nmax cross overlap example: {max_cross_overlap_example}')

        max_self_example_slat1_name = None
        max_self_example_slat2_name = None
        max_cross_example_slat1_name = None
        max_cross_example_slat2_name = None
        for slat_type_set in [horiz_slat_types, vert_slat_types]:
            for slat_type_name in slat_type_set:
                slat_type = slat_type_set[slat_type_name]
                slat_type_domain_list = slat_type.get_domain_label_list()
                if slat_type_domain_list == max_self_example_slat1:
                    max_self_example_slat1_name = slat_type.name
                if slat_type_domain_list == max_self_example_slat2:
                    max_self_example_slat2_name = slat_type.name
                if slat_type_domain_list == max_cross_example_slat1:
                    max_cross_example_slat1_name = slat_type.name
                if slat_type_domain_list == max_cross_example_slat2:
                    max_cross_example_slat2_name = slat_type.name
        if max_self_example_slat1_name is None:
            print(f'ERROR: failed to find max_self_example_slat1')
        if max_self_example_slat2_name is None:
            print(f'ERROR: failed to find max_self_example_slat2')
        if max_cross_example_slat1_name is None:
            print(f'ERROR: failed to find max_cross_example_slat1')
        if max_cross_example_slat2_name is None:
            print(f'ERROR: failed to find max_cross_example_slat2')
        print(f'Max self example slats: {max_self_example_slat1_name} {max_self_example_slat1_name}, cross example slats: {max_cross_example_slat1_name} {max_cross_example_slat2_name}')


        # Do assignment of handles if the num_handles parameter was specified
        if num_handles: # Make a pool of handles of size num_handles and randomly distribute them in place of the glues
            (handled_horiz_slat_types, handled_vert_slat_types) = AssignSlatsFromPool(horiz_slat_types, vert_slat_types, num_handles,
                                                                                      max_error=max_errors, max_iterations=max_iterations,
                                                                                      force_max_error=True)
            if handled_horiz_slat_types == False:
                return
            (horiz_slat_types, vert_slat_types) = (handled_horiz_slat_types, handled_vert_slat_types)

            (max_self_overlap, max_cross_overlap, max_self_overlap_count,
             max_cross_overlap_count, max_self_overlap_example,
             max_cross_overlap_example) = GetIncorrectBindingStatsForSlats(horiz_slat_types, vert_slat_types)

            print(f'max_self_overlap: {max_self_overlap}, max_cross_overlap: {max_cross_overlap}')

        for slat_types in [horiz_slat_types, vert_slat_types]:
            for slat_type in slat_types:
                slat_system.AddPolyominoType(slat_types[slat_type])
                if b_add_flipped_slats == True:
                    rotated_slat_type = slat_types[slat_type].GetRotatedVersion(180)
                    slat_system.AddPolyominoType(rotated_slat_type)
        slat_system_errors = copy.deepcopy(slat_system)

        horiz_slats = []
        vert_slats = []        
        for i in range(cooperativity):
            #slat_type_num = int(i/int(cooperativity/duplicity))
            #horiz_slat = Slat(horiz_slat_types[slat_type_num])
            horiz_slat = Slat(horiz_slat_types[motif[i]])
            horiz_slat.translate((0,i,1))
            horiz_slats.append(horiz_slat)
            slat_system.seed.AddPolyominoForViewingOnly(horiz_slat)

            #vert_slat = Slat(vert_slat_types[slat_type_num])
            vert_slat = Slat(vert_slat_types[motif[i]])
            vert_slat.translate((i,0,0))
            vert_slats.append(vert_slat)
            slat_system.seed.AddPolyominoForViewingOnly(vert_slat)


        #print(f'shifts: {max_self_example_slat1_shift} {max_self_example_slat2_shift} {max_cross_example_slat1_shift} {max_cross_example_slat2_shift}')
        # Make a system showing the example max errors
        max_self_example_slat1_inst = Slat(slat_system_errors.GetPolyominoTypeByName(max_self_example_slat1_name))
        max_self_example_slat2_inst = Slat(slat_system_errors.GetPolyominoTypeByName(max_self_example_slat2_name))
        max_cross_example_slat1_inst = Slat(slat_system_errors.GetPolyominoTypeByName(max_cross_example_slat1_name))
        max_cross_example_slat2_inst = Slat(slat_system_errors.GetPolyominoTypeByName(max_cross_example_slat2_name))
        if max_self_example_slat1_inst.slatType.IsHorizontal() == False:
            max_self_example_slat1_inst = Slat(max_self_example_slat1_inst.slatType.GetRotatedVersion(270))
            max_self_example_slat1_shift = 0 - max_self_example_slat1_shift
        if max_self_example_slat2_inst.slatType.IsHorizontal() == False:
            max_self_example_slat2_inst = Slat(max_self_example_slat2_inst.slatType.GetRotatedVersion(270))
            max_self_example_slat1_shift = 0 - max_self_example_slat1_shift
        if max_cross_example_slat1_inst.slatType.IsHorizontal() == False:
            max_cross_example_slat1_inst = Slat(max_cross_example_slat1_inst.slatType.GetRotatedVersion(270))
            max_cross_example_slat1_shift = 0 - max_cross_example_slat1_shift
        if max_cross_example_slat2_inst.slatType.IsHorizontal() == False:
            max_cross_example_slat2_inst = Slat(max_cross_example_slat2_inst.slatType.GetRotatedVersion(270))
            max_cross_example_slat1_shift = 0 - max_cross_example_slat1_shift

        if max_self_example_slat1_flip == True:
            max_self_example_slat1_inst = Slat(max_self_example_slat1_inst.slatType.GetRotatedVersion(180))
        if max_self_example_slat2_flip == True:
            max_self_example_slat2_inst = Slat(max_self_example_slat2_inst.slatType.GetRotatedVersion(180))
        if max_cross_example_slat1_flip == True:
            max_cross_example_slat1_inst = Slat(max_cross_example_slat1_inst.slatType.GetRotatedVersion(180))
        if max_cross_example_slat2_flip == True:
            max_cross_example_slat2_inst = Slat(max_cross_example_slat2_inst.slatType.GetRotatedVersion(180))

        max_self_example_slat1_inst.translate_reference_point_to((max_self_example_slat1_shift, 0, 0))
        max_self_example_slat2_inst.translate_reference_point_to((max_self_example_slat2_shift, 1, 0))
        max_cross_example_slat1_inst.translate_reference_point_to((max_cross_example_slat1_shift, 3, 0))
        max_cross_example_slat2_inst.translate_reference_point_to((max_cross_example_slat2_shift, 4, 0))

        slat_system_errors.seed.AddPolyominoForViewingOnly(max_self_example_slat1_inst)
        slat_system_errors.seed.AddPolyominoForViewingOnly(max_self_example_slat2_inst)
        slat_system_errors.seed.AddPolyominoForViewingOnly(max_cross_example_slat1_inst)
        slat_system_errors.seed.AddPolyominoForViewingOnly(max_cross_example_slat2_inst)


    ######################################
    # Staggered ribbon growth pattern   # WARNING: the functionality for generating staggered ribbons is significantly out of date!!!!
    else:

        horiz_slat_types = []
        for i in range(duplicity):
            blocks = []
            domains = {}
            for j in range(slat_length):
                k = j % duplicity
                blocks.append((j,0))
                domains[(j,0)] = [(f'{i}-{k}', 'D', 1)]
            slat_type = SlatType(name=f'h{i}', blocks=blocks, domains=domains)
            if i == 0:
                slat_type.color = 'orange'
            horiz_slat_types.append(slat_type)
            slat_system.AddPolyominoType(slat_type)

        vert_slat_types = []
        for i in range(duplicity):
            blocks = []
            domains = {}
            for j in range(slat_length):
                k = j % duplicity
                blocks.append((0,j))
                if j < (slat_length/2):
                    domains[(0,j)] = [(f'{(j-i)%duplicity}-{k}*', 'U', 1)] #[(f'{(j-i)%cooperativity}-{k}*', 'U', 1)]
                else:
                    domains[(0,j)] = [(f'{((j-i)-duplicity)%duplicity}-{k}*', 'U', 1)] #[(f'{((j-i)-cooperativity)%cooperativity}-{k}*', 'U', 1)]
            slat_type = SlatType(name=f'v{i}', blocks=blocks, domains=domains)
            if i == 0:
                slat_type.color = 'red'
            vert_slat_types.append(slat_type)
            slat_system.AddPolyominoType(slat_type)
        print(f'Number of hoizontal slat types: {len(horiz_slat_types)}, number of vertical slat types: {len(vert_slat_types)}')

        horiz_slats = []
        vert_slats = []
        for i in range(cooperativity):
            slat_type_num = i % duplicity #int(i/int(cooperativity/duplicity))
            horiz_slat = Slat(horiz_slat_types[slat_type_num])
            horiz_slat.translate((-i,i,1))
            horiz_slats.append(horiz_slat)
            slat_system.seed.AddPolyominoForViewingOnly(horiz_slat)

            vert_slat = Slat(vert_slat_types[i%len(vert_slat_types)])
            vert_slat.translate((i,-i,0))
            vert_slats.append(vert_slat)
            slat_system.seed.AddPolyominoForViewingOnly(vert_slat)

    overlap_string = ''
    if (not max_self_overlap is None) and (not max_cross_overlap is None):
        overlap_string = f'-self{max_self_overlap}-cross{max_cross_overlap}'

    pdf_file_name = None
    svg_file_name = None
    xml_file_name = f'{directory}/ribbon-{growth_pattern}-coop{cooperativity}-dup{duplicity}{error_string}{overlap_string}.xml'
    pdf_error_example_file_name = None
    svg_error_example_file_name = None
    if b_pdf == True:
        pdf_file_name = f'{directory}/ribbon-{growth_pattern}-coop{cooperativity}-dup{duplicity}{error_string}{overlap_string}.pdf'
        pdf_error_example_file_name = f'{directory}/ribbon-{growth_pattern}-coop{cooperativity}-dup{duplicity}{error_string}{overlap_string}-error-examples.pdf'
    if b_svg == True:
        svg_file_name = f'{directory}/ribbon-{growth_pattern}-coop{cooperativity}-dup{duplicity}{error_string}{overlap_string}.svg'
        svg_error_example_file_name = f'{directory}/ribbon-{growth_pattern}-coop{cooperativity}-dup{duplicity}{error_string}{overlap_string}-error-examples.svg'

    if not os.path.exists(directory):
        os.makedirs(directory)


    slat_system.WriteToFile(xml_file_name, color_mapping=wxPython_color_mapping)
    print(f'Output slat system to {xml_file_name}')

    if (not pdf_file_name is None) or (not svg_file_name is None):
        slat_system.seed.WriteToPDForSVG(pdfFileName=pdf_file_name, svgFileName=svg_file_name, sideLen=50)
        if not slat_system_errors is None:
            slat_system_errors.seed.WriteToPDForSVG(pdfFileName=pdf_error_example_file_name, svgFileName=svg_error_example_file_name, sideLen=50)


if __name__ == "__main__":
    try:
        # starts at the second element of argv since the first one is the script name
        # extraparams are extra arguments passed after all option/keywords are assigned
        # opts is a list containing the pair "option"/"value"
        opts, extraparams = getopt.getopt(sys.argv[1:], "hrpsc:d:o:n:e:i:m:l:f:",
                            ["help", "coop=", "dup=", "rect", "pdf", "svg", "out=", "num=", "errors=", "iter=", "motif=", "listfile=", "flip="])
        #print 'Opts:',opts
        #print 'Extra parameters:',extraparams
    except:
        print("")
        print("Unrecognized option:" + str(sys.argv[1:]))
        DisplayHelp()

    if len(opts) == 0:
        DisplayHelp()

    # Add additional color settings
    extra_slat_color_mapping = {}
    slat_color_mapping.update(extra_slat_color_mapping)

    cooperativity = 4
    duplicity = None
    b_staggered = True
    growth_pattern = 'stag'
    b_pdf = False
    b_svg = False
    directory = '.'
    num_handles = None
    max_errors = None
    max_iterations = 1000
    motif = None
    motif_list = None
    motif_list_file = None
    b_add_flipped_slats = True

    for o,p in opts:
        if o in ['-h', '--help']:
            DisplayHelp()
        elif o in ['-c', '--coop']:
            cooperativity = int(p)
        elif o in ['-d', '--dup']:
            duplicity = int(p)
        elif o in ['-r', '--rect']:
            b_staggered = False
            growth_pattern = 'rect'
        elif o in ['-p', '--pdf']:
            b_pdf = True
        elif o in ['-s', '--svg']:
            b_svg = True
        elif o in ['-o', '--out']:
            directory = p
        elif o in ['-n', '--num']:
            num_handles = int(p)
        elif o in ['-e', '--errors']:
            max_errors = int(p)
        elif o in ['-i', '--iter']:
            max_iterations = int(p)
        elif o in ['-m', '--motif']:
            motif = list(p)
        elif o in ['-l', '--listfile']:
            motif_list_file = p
        elif o in ['-f', '--flip']:
            b_add_flipped_slats = True if p in ['T','t','True','true'] else False
        else:
            DisplayHelp()


    # besties = []
    # with open(motif_list_file, 'r') as f:
    #     lines = f.readlines()
    #     for line in lines:
    #         if line[:len('best slat: ')] == 'best slat: ':
    #             bestie = line[len('best slat: '):].strip()
    #             besties.append(bestie)
    #
    # motif_list_file = motif_list_file[:-4] + '-besties.txt'
    # with open(motif_list_file, 'w') as f:
    #     for bestie in besties:
    #         f.write(f'{bestie}\n')
    # exit()

    if not motif_list_file is None:
        motif_list = []
        with open(motif_list_file, 'r') as f:
            lines = f.readlines()
        for line in lines:
            motif_list.append(line.strip())


    options_dict = {'cooperativity': cooperativity, 'duplicity': duplicity, 'b_staggered': b_staggered, 'growth_pattern': growth_pattern,
                    'b_pdf': b_pdf, 'b_svg': b_svg, 'directory': directory, 'num_handles': num_handles, 'max_errors': max_errors,
                    'max_iterations': max_iterations, 'motif': motif, 'slat_color_mapping': slat_color_mapping, 'motif_list': motif_list,
                    'b_add_flipped_slats': b_add_flipped_slats}
    RibbonSystemGenerator(options_dict)
